# $File: //member/autrijus/DBIx-ReportBuilder/lib/DBIx/ReportBuilder/Clause/Join.pm $ $Author: autrijus $
# $Revision: #1 $ $Change: 7955 $ $DateTime: 2003/09/07 22:39:24 $

package DBIx::ReportBuilder::Clause::Join;
use base 'DBIx::ReportBuilder::Clause';

1;
