# $File: //member/autrijus/DBIx-ReportBuilder/lib/DBIx/ReportBuilder/Part/Graph.pm $ $Author: autrijus $
# $Revision: #5 $ $Change: 7983 $ $DateTime: 2003/09/08 17:03:47 $

package DBIx::ReportBuilder::Part::Graph;
use base 'DBIx::ReportBuilder::Part::Table';

1;
